/******************************************************************************
 * 
 * Copyright 2012 - 2013 Vektor Software.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 *****************************************************************************/
package com.vektorsoft.demux.core.mva;

import java.util.Map;
import java.util.Set;

/**
 * View manager is responsible for managing all views defines for application.
 * It decides which view is to be displayed and provides all information for
 * selected view in order for it to render itself.
 */
public interface DMXViewManager {

    /**
     * Registers specified view with this
     * <code>ViewManager</code>. Registering the view means that
     * <code>ViewManager</code> is responsible for managing it's state.
     *
     * @param view <code>View</code> to register
     */
    void registerView(DMXView view);

    /**
     * Sets selected view as currently active for this
     * <code>ViewManager</code>. Selecting the view will force it to be shown
     * (if not already) and render itself.
     *
     * @param viewId name of the view to be shown
     * @param dataValues data values to set for this view
     */
    void selectView(String viewId, Map<String, Object> dataValues);

    /**
     * Returns a view with specified ID.
     *
     * @param id view ID
     * @return view with specified ID
     */
    DMXView getView(String id);

    /**
     * Returns a list of data IDs that are required by specified view. These are
     * the names under which data are registered with the model.
     *
     * @param viewId ID of the view for which data are required
     * @return list of data IDs
     */
    Set<String> getViewDataIds(String viewId);

    /**
     * Returns a set of view IDs that refer to data with specified ID.
     *
     * @param dataId data ID
     * @return set of view IDs
     */
    Set<String> getViewIdsForData(String dataId);

    /**
     * Forces specified view to render itself due to change in model data.
     * Unlike {@link #selectView()} method, the view does not become active.
     *
     * @param viewId ID of the view to update
     * @param dataValues new data values to set for view
     */
    void updateView(String viewId, Map<String, Object> dataValues);

    /**
     * Updates views that depends on specified data.
     *
     * @param dataMap data map
     */
    void updateViewData(Map<String, Object> dataMap);
}
